/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	spiF429ZI.c		STM32F429 LQFP144 SPI descriptors for spibasic.c

	When		Who	What
	12/04/18	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined spiF4xx_H_
#define spiF4xx_H_
//------------------------------------------------------------------------------

/*
STM32F427xx and STM32F429xx

		MOSI	MISO	SCK		NSS		APB

SPI_1	PA7		PA6		PA5		PA4		2
		PB5		PB4		PB3=SWO	PA15

SPI_2	PB15	PB14	PB13	PB12	1
		PI3		PI2		PI1		PI0
		PC3		PC2		PD3
						PB10	PB9

SPI_3	PD6		(PC11)	(PC10)	(PA4)	1
		(PB5)	(PB4)	(PB3)	(PA15)
		(PC12)

SPI_4	PE6		PE5		PE2		PE4		2
		PE14	PE13	PE12	PE11

SPI_5	PF9		PF8		PF7		PF6		2
		PF11	PH7		PH6		PH5

SPI_6	PG14	PG12	PG13	PG8		2

All pins in AF5 except () wich are AF6
PIx and PHx only if pin count > 169
PB3 is also SWO

SPI4, SPI5, SPI6 defined only on F429/439 and 427/437
*/

/*
	Mode	CPOL	CPHA
	0		0		0
	1		0		1
	2		1		0
	3		1		1

	CPOL	Clock polarity controls the steady state value of the clock when no data is being transferred
			0: Low level when idle state

	CPHA	Controls which edge of the clock locks the data.
			0:first edge, 1:second edge
*/

static	const spiDesc_t	spiDescArray [] =
{
	#if (WITH_SPI_1 == 1)
// Master SPI test
	{
		10,								// id
		RCC_APB2ENR_SPI1EN_Pos,			// SPI periph pos
		SPI1,							// Pointer to SPI registers
		{	'B',	5,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH   },		// MOSI
		{	'B',	4,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH   },		// MISO
		{	'A',	5,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH   },		// SCK
//		{	'A',	4,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// NSS select AUTO/HARD
		{	'A',	4,	AA_GPIO_AF_0,	AA_GPIO_MODE_OUTPUT_PP | AA_GPIO_SPEED_MEDIUM },	// NSS select USER/SOFT
		{	// DMA Tx
			2,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (3, 3),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
		{	// DMA Rx
			2,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (2, 3),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
	},
	#endif	// WITH_SPI_1
/*
	#if (WITH_SPI_1 == 1)
	{
		11,
		RCC_APB2ENR_SPI1EN_Pos,
		SPI1,
		{	'A',	7,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// MOSI
		{	'A',	6,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// MISO
		{	'A',	5,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// SCK
		{	'A',	4,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// NSS
		{	// DMA Tx
			0,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (0, 0),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
		{	// DMA Rx
			0,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (0, 3),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
	},
	#endif	// WITH_SPI_1
*/
	#if (WITH_SPI_2 == 1)
// Slave SPI test
	{
		20,
		RCC_APB1ENR_SPI2EN_Pos,
		SPI2,
		{	'B',	15,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP    | AA_GPIO_SPEED_HIGH },		// MOSI
		{	'B',	14,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP    | AA_GPIO_SPEED_HIGH },		// MISO
		{	'B',	13,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP_DN | AA_GPIO_SPEED_HIGH },		// SCK
//		{	'B',	12,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP    | AA_GPIO_SPEED_MEDIUM },	// NSS
		{	'B',	12,	AA_GPIO_AF_0,	AA_GPIO_MODE_INPUT_UP  | AA_GPIO_SPEED_MEDIUM },	// NSS
		{	// DMA Tx
			1,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (4, 0),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
		{	// DMA Rx
			1,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (3, 0),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
	},
	#endif	// WITH_SPI_2

	#if (WITH_SPI_3 == 1)
	{
		30,
		RCC_APB1ENR_SPI3EN_Pos,
		SPI3,
		{	'D',	6,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// MOSI
		{	'C',	11,	AA_GPIO_AF_6,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// MISO
		{	'C',	10,	AA_GPIO_AF_6,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// SCK
		{	'A',	4,	AA_GPIO_AF_6,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// NSS
		{	// DMA Tx
			0,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (0, 0),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
		{	// DMA Rx
			0,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (0, 3),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
	},
	#endif	// WITH_SPI_3

	#if (WITH_SPI_4 == 1  &&  defined SPI4)
	{
		40,
		RCC_APB2ENR_SPI4EN_Pos,
		SPI4,
		{	'E',	6,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// MOSI
		{	'E',	5,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// MISO
		{	'E',	2,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// SCK
		{	'E',	4,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// NSS
		{	// DMA Tx
			0,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (0, 0),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
		{	// DMA Rx
			0,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (0, 3),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
	},
	#endif	// WITH_SPI_4

	#if (WITH_SPI_5 == 1  &&  defined SPI5)
	{
		50,
		RCC_APB2ENR_SPI5EN_Pos,
		SPI5,
		{	'F',	9,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// MOSI
		{	'F',	8,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// MISO
		{	'F',	7,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// SCK
		{	'F',	6,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// NSS
		{	// DMA Tx
			0,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (0, 0),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
		{	// DMA Rx
			0,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (0, 3),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
	},
	#endif	// WITH_SPI_5

	#if (WITH_SPI_6 == 1  &&  defined SPI6)
	{
		60,
		RCC_APB2ENR_SPI6EN_Pos,
		SPI6,
		{	'G',	14,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// MOSI
		{	'G',	12,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// MISO
		{	'G',	13,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },		// SCK
		{	'G',	8,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },	// NSS
		{	// DMA Tx
			0,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (0, 0),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
		{	// DMA Rx
			0,									//	DMA number 1 or 2, 0 => unused descriptor
			DMA_TO_STREAMCHAN (0, 3),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
			SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			LL_DMA_MODE_NORMAL                |	//	Mode
			LL_DMA_PRIORITY_HIGH,				//	DMA priority
		},
	},
	#endif	// WITH_SPI_6
} ;

//------------------------------------------------------------------------------
#endif	// spiF4xx_H_
