/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	testadc.c	ADC basic API tests

	When		Who	What
	25/05/18	ac	Creation

----------------------------------------------------------------------
*/

#include	"test.h"
#include	"aaprintf.h"
#include	"aalogmes.h"
#include	"adcbasic.h"
#include	"timbasic.h"
#include	"dmabasic.h"

//----------------------------------------------------------------------
//	ADC tests

#define	TADC_CHANNEL_COUNT	1

static	const uint8_t			adcChannels [TADC_CHANNEL_COUNT] = { 3u } ;	// PA3
static	const adcSamplingTime_t	samplingTime1 [1] =
{
	{ 3u, LL_ADC_SAMPLINGTIME_480CYCLES },		// ADC cycles count: 7=>480
} ;

#define	ADCX	ADC_ADC1

//----------------------------------------------------------------------
//	Unitary conversion on software trigger

static	void	adcTest1 (void) ;

static	void	adcTest1 (void)
{
	adcDesc_t	* pAdcDesc ;
	uint16_t	data16 ;

	// Configure 1 channel
	adcInit				(ADCX, ADC_PRES4, 0) ;						// ADC common parameters
	adcInitChannels 	(ADCX, adcChannels, TADC_CHANNEL_COUNT) ;	// Configure analog pins
//	adcInitAdc 			(ADCX | ADC_INT_EOCS, & pAdcDesc) ;	// Initialize ADC with interrupt
	adcInitAdc 			(ADCX, & pAdcDesc) ;	// Initialize ADC
	adcInitSequencer	(pAdcDesc, ADC_ONESHOT, adcChannels, TADC_CHANNEL_COUNT) ;
	adcInitSamplingTime	(pAdcDesc, samplingTime1, sizeof (samplingTime1) / sizeof (adcSamplingTime_t)) ;
	adcSetTrigger 		(pAdcDesc, LL_ADC_REG_TRIG_SOFTWARE, 0) ;
	adcStart			(pAdcDesc) ;

	while (1)
	{
		adcSwTrigger (pAdcDesc) ;
		while (adcWaitForEoc (pAdcDesc) != ADC_ENONE)
		{
		}
		adcReadUnitary	(pAdcDesc, & data16) ;
		aaPrintf ("ADC: %04X\n", data16) ;

		aaTaskDelay (100) ;

		if (aaCheckGetChar () != 0)
		{
			break ;		// Test cancelled by the user
		}
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Sequence conversion on software trigger

static	void	adcTest2Callback	(adcDesc_t * pAdcDesc, uint32_t event, uintptr_t arg) ;
static	void	adcDmaCallback		(uint32_t event, uint32_t source, uintptr_t arg) ;
static	void	adcTest2			(void) ;

#define	TMAX_CONV	8

static	uint8_t	adcSequence [TMAX_CONV] = { 3, 3, 3, 3,  3, 3, 3, 3} ;

static	const adcSamplingTime_t	samplingTime2 [1] =
{
	{ 3, LL_ADC_SAMPLINGTIME_480CYCLES },		// ADC cycles count: 7=>480
} ;

//----------------------------------------------------------------------
//	ADC interrupt callback

static	void	BSP_ATTR_USED adcTest2Callback (adcDesc_t * pAdcDesc, uint32_t event, uintptr_t arg)
{
	aaLogMes ("adcTest2Callback %d\n", event, 0, 0, 0, 0) ;
	if (event == ADC_EVENT_EOC)
	{
		aaSignalSend ((aaTaskId_t) arg, AA_SIGNAL_IO) ;
	}

	(void) pAdcDesc ;
}

//----------------------------------------------------------------------
//	DMA interrupt callback. Used by test2() and test3()

static	void adcDmaCallback (uint32_t event, uint32_t source, uintptr_t arg)
{
	if (event == DMA_EV_COMPLETE)
	{
		aaSignalSend ((aaTaskId_t) arg, AA_SIGNAL_IO) ;
	}

	(void) source ;
}

//----------------------------------------------------------------------
// Convert a sequence of channels on ADC1
// When using DMA, EOC flag of SR register is cleared by DMA
// So use ADC_INT_EOC flag of adcInitAdc() and adcSetCallback() only if DMA is not used

static	void	adcTest2 (void)
{
	adcDesc_t		* pAdcDesc ;
	dmaDesc_t		* pDmaDesc ;
	uint16_t		data16 [TMAX_CONV] ;

	aaPrintf ("ADC Test 2\n", 0, 0, 0, 0, 0) ;

	// Configure ADC with SCAN and software trigger
	adcInit				(ADCX, ADC_PRES4, 0) ;							// ADC common parameters
	adcInitChannels 	(ADCX, adcChannels, TADC_CHANNEL_COUNT) ;		// Configure analog pins
//	adcInitAdc 			(ADCX | ADC_INT_EOC, & pAdcDesc) ;				// Initialize ADC
//	adcInitAdc 			(ADCX | ADC_RESO8BITS, & pAdcDesc) ;			// Initialize ADC
	adcInitAdc 			(ADCX, & pAdcDesc) ;							// Initialize ADC
	adcInitSequencer	(pAdcDesc, ADC_ONESHOT, adcSequence, TMAX_CONV) ;
	adcInitSamplingTime	(pAdcDesc, samplingTime2, sizeof (samplingTime2) / sizeof (adcSamplingTime_t)) ;
	adcSetTrigger 		(pAdcDesc, LL_ADC_REG_TRIG_SOFTWARE, 0) ;
//	adcSetCallback		(pAdcDesc, adcTest2Callback, aaTaskSelfId ()) ;		// Only if ADC_INT_EOC is used

	// Configure DMA and callback
	adcDmaInit			(pAdcDesc, ADC_DMA_CIRCULAR, TMAX_CONV) ;
	adcDmaBuffer		(pAdcDesc, data16, NULL) ;
	(void) adcDmaGetDesc (pAdcDesc, & pDmaDesc) ;
	dmaSetCallback		(pDmaDesc, adcDmaCallback, aaTaskSelfId ()) ;

	adcStart			(pAdcDesc) ;

	while (1)
	{
		aaSignalClear (AA_SELFTASKID, AA_SIGNAL_IO) ;
		adcSwTrigger (pAdcDesc) ;

		aaSignalWait (AA_SIGNAL_IO, NULL, AA_SIGNAL_OR, AA_INFINITE) ;
//		while (adcWaitForEoc (pAdcDesc) != ADC_ENONE)
//		{
//		}
		aaPrintf ("ADC: %04X %04X %04X %04X %04X %04X %04X %04X  %04X\n",
				data16[0], data16[1], data16[2], data16[3], data16[4], data16[5], data16[6], data16[7],
				(data16[0] + data16[1] + data16[2] + data16[3] + data16[4] + data16[5] + data16[6] + data16[7]) / 8) ;

		aaTaskDelay (100) ;

		if (aaCheckGetChar () != 0)
		{
			break ;		// Test cancelled by the user
		}
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Dual ADC1+ADC2, TIM trigger

#if (defined ADC2)

static	void 	adcTest3Callback	(adcDesc_t * pAdcDesc, uint32_t event, uintptr_t arg) ;
static	void	adcTest3			(void) ;


#define	ADC_TIMER		3

#define	DUALMAX_CONV	8

static	const uint8_t			dualChannels [2] = { 3, 7 } ;	// PA3, PA7
static	const adcSamplingTime_t	dualSampling [2] =
{
	{ 3, LL_ADC_SAMPLINGTIME_480CYCLES },		// ADC cycles count: 7=>480
	{ 7, LL_ADC_SAMPLINGTIME_480CYCLES}
} ;
static	const uint8_t			dualSeqAdc1  [DUALMAX_CONV] = { 3, 3, 3, 3, 3, 3, 3, 3 } ;
static	const uint8_t			dualSeqAdc2  [DUALMAX_CONV] = { 7, 7, 7, 7, 7, 7, 7, 7 } ;

static	adcDesc_t				* pAdcDesc1 ;
static	adcDesc_t				* pAdcDesc2 ;

static	uint32_t				dualData32 [DUALMAX_CONV] ;		// ADC2 in upper half-word, ADC1 in lower half-word

//----------------------------------------------------------------------

static	void	BSP_ATTR_USED adcTest3Callback (adcDesc_t * pAdcDesc, uint32_t event, uintptr_t arg)
{
	aaLogMes ("adcTest3Callback %d\n", event, 0, 0, 0, 0) ;
	if (event == ADC_EVENT_EOC)
	{
		aaSignalSend ((aaTaskId_t) arg, AA_SIGNAL_IO) ;
	}

	(void) pAdcDesc ;
	(void) arg ;
}

//----------------------------------------------------------------------

static	void	adcTest3 (void)
{
	const timDesc_t		* pTimDesc ;
	dmaDesc_t			* pDmaDesc ;
	aaSignal_t			sigsOut ;

	aaPrintf ("ADC Test 3\n", 0, 0, 0, 0, 0) ;

	// Configure ADC with SCAN and software trigger
	adcInit				(ADC_DUAL, ADC_PRES4, 0) ;						// ADC common parameters
	adcInitChannels 	(ADC_ADC1, dualChannels, sizeof (dualChannels) / sizeof (uint8_t)) ;	// Configure analog pins, common to ADC1 and ADC2

	// Initialize ADC1
	adcInitAdc 			(ADC_ADC1 | ADC_INT_EOC, & pAdcDesc1) ;
	adcInitSequencer	(pAdcDesc1, ADC_ONESHOT, dualSeqAdc1, DUALMAX_CONV) ;
	adcInitSamplingTime	(pAdcDesc1, dualSampling, sizeof (dualSampling) / sizeof (adcSamplingTime_t)) ;
	adcSetTrigger 		(pAdcDesc1, LL_ADC_REG_TRIG_EXT_TIM3_TRGO, LL_ADC_REG_TRIG_EXT_RISING) ;

	// Initialize ADC2
	adcInitAdc 			(ADC_ADC2, & pAdcDesc2) ;
	adcInitSequencer	(pAdcDesc2, ADC_ONESHOT, dualSeqAdc2, DUALMAX_CONV) ;
	adcInitSamplingTime	(pAdcDesc2, dualSampling, sizeof (dualSampling) / sizeof (adcSamplingTime_t)) ;

	// Configure DMA and callback
	adcDmaInit			(pAdcDesc1, ADC_DMA_CIRCULAR, TMAX_CONV) ;
	adcDmaBuffer		(pAdcDesc1, dualData32, NULL) ;
	(void) adcDmaGetDesc (pAdcDesc1, & pDmaDesc) ;
	dmaSetCallback		(pDmaDesc, adcDmaCallback, aaTaskSelfId ()) ;

	// Configure the timer
	if (timGetDesc (ADC_TIMER, & pTimDesc) == TIM_EFAIL)
	{
		aaPrintf ("Timer ADC: Unknown descriptor id %u\n", ADC_TIMER) ;
		return ;
	}
	timTimeBaseInit     (pTimDesc, 0, 10, 0) ;		// 10 Hz
	timSetTriggerOutput (pTimDesc, LL_TIM_TRGO_UPDATE) ;		// Set timer TRGO

	adcStart	(pAdcDesc1) ;
	adcStart	(pAdcDesc2) ;

	// The DMA callback will use aaSignalSend() to awake the task
	aaSignalClear (AA_SELFTASKID, AA_SIGNAL_IO) ;
	timStart (pTimDesc) ;

	while (1)
	{
		aaSignalWait (AA_SIGNAL_IO, & sigsOut, AA_SIGNAL_OR, AA_INFINITE) ;
		aaPrintf ("ADC: %08X %08X %08X %08X %08X %08X %08X %08X\n",
				dualData32[0], dualData32[1], dualData32[2], dualData32[3],
				dualData32[4], dualData32[5], dualData32[6], dualData32[7]) ;

		if (aaCheckGetChar () != 0)
		{
			timStop (pTimDesc) ;
			break ;		// Test cancelled by the user
		}
	}
}
#endif

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Get values from internal sensors, F42x / F43x
//	Temp/VBAT: ADC1_IN18, Internal REF: ADC1_IN17
//	temp = {(VSENSE - V25) / Avg_Slope} + 25
//	For ST32F429: V25=0.76V    Avg_Slope=2.5mv/dC

//	Using calibration values TS_CAL1 and TS_CAL2:
//	Temp  = 80 / (TS_CAL2 - TS_CAL1) x (VSENSE - TS_CAL1) + 30


static	const uint8_t			adcChannelT = 18 ;	// Temp sensor
static	const uint8_t			adcChannelR = 17 ;	// Internal REF

static	const adcSamplingTime_t	samplingTimeT [2] =
{
	{ 17, LL_ADC_SAMPLINGTIME_480CYCLES },		// ADC cycles count: 7=>480
	{ 18, LL_ADC_SAMPLINGTIME_480CYCLES },		// ADC cycles count: 7=>480
} ;

//----------------------------------------------------------------------

static	void	adcTestT (void) ;

static	void	adcTestT (void)
{
	adcDesc_t	* pAdcDescT ;
	uint16_t	data16 ;
	int32_t		val ;

	// Configure the temperature sensor channel
	adcInit				(ADC_ADC1, ADC_PRES4, 0) ;		// ADC common parameters
	adcInitChannels 	(ADC_ADC1, & adcChannelT, 1) ;	// Configure analog pins
	adcInitAdc 			(ADC_ADC1, & pAdcDescT) ;		// Initialize ADC
	adcSensorEnable		(pAdcDescT, ADC_SENSOR_TEMP) ;
	adcInitSamplingTime	(pAdcDescT, samplingTimeT, sizeof (samplingTimeT) / sizeof (adcSamplingTime_t)) ;
//	adcInitSequencer	(pAdcDescT, ADC_ONESHOT, & adcChannelT, 1) ;
	adcSetTrigger 		(pAdcDescT, LL_ADC_REG_TRIG_SOFTWARE, 0) ;
	adcStart			(pAdcDescT) ;

	aaPrintf ("Temp Cal parameters: %u %u, %u %u\n", TEMPSENSOR_CAL1_TEMP, * TEMPSENSOR_CAL1_ADDR, TEMPSENSOR_CAL2_TEMP, *TEMPSENSOR_CAL2_ADDR) ;

	while (1)
	{
		// Temperature
		adcInitSequencer	(pAdcDescT, ADC_ONESHOT, & adcChannelT, 1) ;
		adcSensorEnable		(pAdcDescT, ADC_SENSOR_TEMP) ;
		bspDelayUs (20) ;
		adcSwTrigger (pAdcDescT) ;
		while (adcWaitForEoc (pAdcDescT) != ADC_ENONE)
		{
		}
		adcReadUnitary	(pAdcDescT, & data16) ;
		val = adcToTemp (data16) ;
		aaPrintf ("Temp: %04X %4d.%d dC  ", data16, val/10, val % 10) ;

		// VBAT
		adcInitSequencer	(pAdcDescT, ADC_ONESHOT, & adcChannelT, 1) ;
		adcSensorEnable		(pAdcDescT, ADC_SENSOR_VBAT) ;
		bspDelayUs (20) ;
		adcSwTrigger (pAdcDescT) ;
		while (adcWaitForEoc (pAdcDescT) != ADC_ENONE)
		{
		}
		adcReadUnitary	(pAdcDescT, & data16) ;
		val = ((data16 * 33000) / 4096) * 4 ;		// VBAT is divided by 4 prior ADC conversion
		aaPrintf ("VBAT: %04X %4d.%d mV  ", data16, val/10, val % 10) ;

		// REF Int
		adcInitSequencer	(pAdcDescT, ADC_ONESHOT, & adcChannelR, 1) ;
		adcSensorEnable		(pAdcDescT, ADC_SENSOR_TEMP) ;
		bspDelayUs (20) ;
		adcSwTrigger (pAdcDescT) ;
		while (adcWaitForEoc (pAdcDescT) != ADC_ENONE)
		{
		}
		adcReadUnitary	(pAdcDescT, & data16) ;
		val = (data16 * 3300) / 4096 ;		// Convert to mV
		aaPrintf ("Ref: %04X %4d mV\n", data16, val) ;

		aaTaskDelay (500) ;

		if (aaCheckGetChar () != 0)
		{
			adcSensorEnable (pAdcDescT, ADC_SENSOR_NONE) ;
			break ;		// Test cancelled by the user
		}
	}
}

//----------------------------------------------------------------------

void	adcTest (void)
{
	uint32_t			cc ;

	while (1)
	{
		aaPrintf ("ADC test, h for menu help\n") ;
		cc = aaGetChar () ;
		aaPutChar ((char) cc) ;
		aaPutChar ('\n') ;

		if (cc == 'h')
		{
			aaPrintf ("1    Unitary conversion on software trigger\n") ;
			aaPrintf ("2    Sequence conversion on software trigger\n") ;
#if (defined ADC2)
			aaPrintf ("3    Dual ADC1 + ADC2, TIM3 trigger\n") ;
#endif
			aaPrintf ("t    Temperature sensor\n") ;
			aaPrintf ("q    Quit test\n") ;
			continue ;
		}

		if (cc == 'q')
		{
			break ;	// End of test
		}

		switch (cc)
		{
			case '1':
				adcTest1 () ;
				break ;

			case '2':
				adcTest2 () ;
				break ;

#if (defined ADC2)
			case '3':
				adcTest3 () ;
				break ;
#endif

			case 't':
				adcTestT () ;
				break ;

			default:
				break ;
		}
		if (aaCheckGetChar () != 0)
		{
			(void) aaGetChar () ;
		}
	}
}

//----------------------------------------------------------------------
