/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	testadc.c	ADC basic API tests

	When		Who	What
	25/05/18	ac	Creation

----------------------------------------------------------------------
*/

#include	"test.h"

#include	"aausb.h"
#include	"stdlib.h"		// for strtoul
#include	"string.h"		// for strtok_r

//--------------------------------------------------------------------------------
//	Test USB performance

static	cdctHandle_t	hCdc ;

#define	USB_SIZE		1000u
static	uint8_t			usbData [USB_SIZE] ;
static	uint32_t		usbSize ;
static	uint32_t		usbLoop ;
static	uint32_t		usbConsole ;	// 1 If the console is on USB
static	aaDevIo_t		usbDevice ;

void	usbTest (void)
{
	uint32_t			cc ;

	// Initialize parameters on 1st call
	if (usbLoop == 0u)
	{
		usbLoop = 10u ;
		usbSize = 1000u ;
		aaMEMSET (usbData, 'A', USB_SIZE) ;
	}

	aaPrintf ("\nUSB test, i to initialize USB stack\n\n") ;

	while (1)
	{
		aaPrintf ("USB test, h for menu help\n") ;
		cc = aaGetChar () ;
		aaPutChar ((char) cc) ;

		if (cc == 'h')
		{
			aaPutChar ('\n') ;
			aaPrintf ("i     Initialize\n") ;
			aaPrintf ("s     Put string\n") ;
			aaPrintf ("w     Write data buffer\n") ;
			aaPrintf ("l n   Set loop count\n") ;
			aaPrintf ("z n   Set data size (1..%u)\n", USB_SIZE) ;
			aaPrintf ("c     Switch console UART/USB\n") ;
			aaPrintf ("0     Use CDC 0\n") ;
			aaPrintf ("1     Use CDC 1\n") ;
			aaPrintf ("?     print parameter\n") ;
			aaPrintf ("q     Quit test\n") ;
			continue ;
		}

		if (cc == 'q')
		{
			break ;	// End of test
		}

		if (cc == 'l'  ||  cc == 'z')
		{
			char			cmd [32] ;
			char			* pStr, * pCmd ;
			uint32_t		val ;

			aaPutChar (' ') ;
			aaGets (cmd, sizeof (cmd)) ;
			pCmd = strtok_r (cmd, " \t", & pStr);
			if (pCmd == NULL)
			{
				continue ;
			}

			val = strtoul (pCmd, NULL, 0) ;
			if (val != 0u)
			{
				if (cc == 'l')
				{
					usbLoop = val ;
				}
				else
				{
					usbSize = val ;
				}
			}
		}

		aaPutChar ('\n') ;
		switch (cc)
		{
			case 'i':
				aaUsbInit () ;
				hCdc = cdcGethandle (0);
				break ;

			case '0':		// Use CDC 0
				hCdc = cdcGethandle (0);
				if (hCdc == NULL)
				{
					aaPrintf ("Invalid CDC 0\n") ;
				}
				break ;

			case '1':		// Use CDC 1
				hCdc = cdcGethandle (1);
				if (hCdc == NULL)
				{
					aaPrintf ("Invalid CDC 1\n") ;
				}
				break ;

			case 's':
				if (hCdc != NULL)
				{
					cdcPutString (hCdc, "012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789\r\n") ;
				}
				else
				{
					aaPrintf ("USB not initialized\n") ;
				}
				break ;

			case 'w':
				{
					if (hCdc != NULL)
					{
						uint32_t	ii ;
						uint32_t	now ;

						now = bspTsGet () ;
						for (ii = 0 ; ii < usbLoop ; ii++)
						{
							cdcWrite (hCdc, usbData, usbSize, 0) ;	// Without flush
						}
						cdcWrite (hCdc, NULL, 0, 1) ;	// Flush
						now = bspTsDelta (& now) ;
						ii = (((usbLoop * usbSize) / 100u) * 1000000u) / now ;
						aaPrintf ("%6u: %6u us - %3u.%01u KB/s\n", usbLoop * usbSize, now, ii / 10u, ii % 10u) ;
					}
					else
					{
						aaPrintf ("USB not initialized\n") ;
					}
				}
				break ;

			case 'c':
				if (hCdc != NULL)
				{
					if (usbConsole == 0u)
					{
						// Switch to USB console
						cdcSetOption (hCdc, USB_DEV_CRLF, 1) ;		// Convert \n to \r\n
						cdcInitStdDev (& usbDevice, hCdc) ;			// Initialize the devIo with USB functions
						aaInitStdIn	  (& usbDevice) ;				// Use this devIo for input
						aaInitStdOut  (& usbDevice) ;				// Use this devIo for output
						usbConsole = 1u ;
					}
					else
					{
						// Switch to UART console, if UART console is defined
						aaDevIo_t	* hDevice = aaGetDefaultConsole () ;
						if (hDevice != NULL)
						{
							// Restore the default console
							aaInitStdIn	 (hDevice) ;
							aaInitStdOut (hDevice) ;
						}
						else
						{
							// No default console !!!!
							aaInitStdIn	 (NULL) ;
							aaInitStdOut (NULL) ;
						}
						usbConsole = 0u ;
					}
					bspBanner_ () ;	// Display something
				}
				break ;

			case '?':
				aaPrintf ("Size     %u\n", usbSize) ;
				aaPrintf ("Loop     %u\n\n", usbLoop) ;
				break ;

			default:
				break ;
		}
	}
}

//--------------------------------------------------------------------------------
